package cloning;

public class Mouse implements Cloneable {
    private String type;
    private int xPos, yPos;

    public Mouse(String type) {
        this.type = type;
        xPos = yPos = 0;
    }

    public void moveMouse(int xPos, int yPos) {
        this.xPos = xPos;
        this.yPos = yPos;
    }

    @Override
    public String toString() { 
        return type + " -> xPos: " + xPos + ", yPos: " + yPos;
    }

    /* The clone() method creates a shallow copy of the Mouse object.
     * 
     * - The return type is `Mouse` instead of `Object`, which is allowed 
     *   because `Mouse` is a **covariant return type** (subclass of Object).
     * - The call to `super.clone()` performs a **shallow copy**.
     * - Since all fields in Mouse are **primitives or immutable (String)**, 
     *   this shallow copy is sufficient. 
     * - If we had a **mutable reference field** (e.g., an array or a List), 
     *   we would need to manually clone it for a **deep copy**.
     */
    @Override
    public Mouse clone() {
        try {
            return (Mouse) super.clone(); // Shallow copy is fine since fields are primitives/immutable
        } catch (CloneNotSupportedException e) {
            throw new AssertionError("Cloning not supported, but Cloneable is implemented!", e);
        }
    }
}
